(async function () {
    await ShanhaiBI.initSettings({
        "format": [
            {
                name: "model",
                alias: "模型设置",
                group: "plugin_model"
            },
            {
                name: "model-file",
                alias: "模型文件",
                type: "model-file",
                default: "./resource/horse.glb",
                group: "plugin_model"
            },
            {
                name: "light-intensity",
                alias: "光照强度",
                type: "number",
                default: 5,
                group: "plugin_model"
            },
            {
                name: "camera-farthest-perspective",
                alias: "镜头最远视角",
                type: "number",
                default: 750,
                group: "plugin_model"
            },
            {
                name: "camera-recent-perspective",
                alias: "镜头最近视角",
                type: "number",
                default: 20,
                group: "plugin_model"
            },
            {
                name: "model-static",
                alias: "静止",
                type: "boolean",
                default: false,
                group: "plugin_model"
            },
            {
                name: "title",
                alias: "标题设置",
                type: "boolean",
                default: "true",
                group: "plugin_title"
            },
            {
                name: "title-size",
                alias: "字体大小",
                type: "number",
                default: 12,
                group: "plugin_title"
            },
            {
                name: "title-color",
                alias: "文本颜色",
                type: "color",
                default: "#ff0000",
                group: "plugin_title"
            },
            {
                name: "title-position",
                alias: "标题位置",
                type: "select",
                choices: [
                    {
                        label: "左侧偏上",
                        value: "left-top",
                    },
                    {
                        label: "左侧居中",
                        value: "left-middle",
                    },
                    {
                        label: "左侧偏下",
                        value: "left-bottom",
                    },
                    {
                        label: "右侧偏上",
                        value: "right-top",
                    },
                    {
                        label: "右侧居中",
                        value: "right-middle",
                    },
                    {
                        label: "右侧偏下",
                        value: "right-bottom",
                    },
                    {
                        label: "顶部居中",
                        value: "top-center",
                    },
                    {
                        label: "底部居中",
                        value: "bottom-center",
                    },
                ],
                default: "left-top",
                group: "plugin_title"
            }
        ]
    })
    // 标题样式
    let titleDisplay = await ShanhaiBI.getSetting("title");
    $("#title").css({
        position: "absolute",
        display: titleDisplay ? "block" : "none",
        color: await ShanhaiBI.getSetting("title-color"),
        fontSize: await ShanhaiBI.getSetting("title-size"),
        ...getPosition(await ShanhaiBI.getSetting("title-position"))
    });
    let modelUrl = await ShanhaiBI.getSetting("model-file");
    if (!modelUrl) return;
    //绘制模型
    console.log( await ShanhaiBI.getSetting("title-color"))
    const container = $("#container")[0];
    let width = $("html")[0].offsetWidth;
    let height = $("html")[0].offsetHeight;
    let camera_farthest_perspective = await ShanhaiBI.getSetting("camera-farthest-perspective");
    let camera_recent_perspective = await ShanhaiBI.getSetting("camera-recent-perspective");
    let model_static = await ShanhaiBI.getSetting("model-static");

    const scene = new THREE.Scene();  //场景
    const camera = new THREE.PerspectiveCamera(45, width / height, 0.1, 1000); //相机
    camera.position.z = setPositionZ(height);

    //渲染器
    let renderer = new THREE.WebGLRenderer({ alpha: true, antalias: true });
    renderer.setSize(width, height);  //设置窗口尺寸
    container.appendChild(renderer.domElement);  //将渲染器加载到dom中

    //灯光
    let light = new THREE.AmbientLight();  //环境光
    light.position.set(0, height, 0);
    light.intensity = await ShanhaiBI.getSetting("light-intensity");
    scene.add(light);

    let mesh, mixer;
    let prevTime = Date.now();
    new Promise((resolve) => {
        new THREE.GLTFLoader().load(modelUrl, function (gltf) {
            gltf.scene.name = "3dmodel";
            mesh = gltf.scene;
            scene.add(gltf.scene);
            resolve();
            if(!model_static) {
                mixer = new THREE.AnimationMixer(mesh);
                mixer.clipAction(gltf.animations[0]).setDuration(1).play();
            }
        }, undefined, err => console.log(err));
    }).then(() => {
        mesh.rotation.x = 0.7; //正面
        render();
    })
    function render() {
        //渲染函数
        requestAnimationFrame(render);
        if (mixer) {
            const time = Date.now();
            mixer.update((time - prevTime) * 0.001);
            prevTime = time;
        }
        renderer.render(scene, camera);
    }
    function setPositionZ(position_z) {
        if(position_z < camera_recent_perspective) return camera_recent_perspective;
        if(position_z > camera_farthest_perspective) return camera_farthest_perspective;
        return position_z;
    }

    function onMousemove(ev) {
        //图形旋转
        let angle = 0.1;
        if (ev.movementX > 0) {
            mesh.rotation.y += angle;
        } else if (ev.movementX < 0) {
            mesh.rotation.y -= angle;
        }
        if (ev.movementY > 0) {
            mesh.rotation.x += angle;
            if (mesh.rotation.x > 1.6) mesh.rotation.x = 1.6;
        } else if (ev.movementY < 0) {
            mesh.rotation.x -= angle;
            if (mesh.rotation.x < -1.6) mesh.rotation.x = -1.6;
        }
    }
    document.addEventListener("mousedown", ev => {
        document.addEventListener("mousemove", onMousemove);
    })
    document.addEventListener("mouseup", ev => {
        document.removeEventListener("mousemove", onMousemove);
    })
    //放大缩小
    document.addEventListener("mousewheel", ev => {
        if (ev.deltaY > 0) {
            camera.position.z += 20;
            if (camera.position.z > camera_farthest_perspective) camera.position.z = camera_farthest_perspective;
        } else {
            camera.position.z -= 20;
            if (camera.position.z < camera_recent_perspective) camera.position.z = camera_recent_perspective;
        }
    })
    function getPosition(value) {
        let position = {};
        if (value === "left-top") {
            position = { left: 0, top: 0 };
        } else if (value === "left-middle") {
            position = { left: 0, top: "50%", transform: "translateY(-50%)" };
        } else if (value === "left-bottom") {
            position = { left: 0, bottom: 0 };
        } else if (value === "right-top") {
            position = { right: 0, top: 0 };
        } else if (value === "right-middle") {
            position = { right: 0, top: "50%", transform: "translateY(-50%)" };
        } else if (value === "right-bottom") {
            position = { right: 0, bottom: 0 };
        } else if (value === "top-center") {
            position = { top: 0, left: "50%", transform: "translateX(-50%)" };
        } else if (value === "bottom-center") {
            position = { bottom: 0, left: "50%", transform: "translateX(-50%)" };
        }
        return position;
    }
})();




